-- UI Setup --

--Create context for rendering
local context = UI.CreateContext("Context")

local slideDownFrame = UI.CreateFrame("Frame", "slideDownFrame", context)

local time_5m_marker = UI.CreateFrame("Text", "time_5m_marker", slideDownFrame)
local time_1m_marker = UI.CreateFrame("Text", "time_1m_marker", slideDownFrame)
local time_30s_marker = UI.CreateFrame("Text", "time_30s_marker", slideDownFrame)
local time_15s_marker = UI.CreateFrame("Text", "time_15s_marker", slideDownFrame)
local time_5s_marker = UI.CreateFrame("Text", "time_5s_marker", slideDownFrame)
local time_1s_marker = UI.CreateFrame("Text", "time_1s_marker", slideDownFrame)

local label_marker = { 
	["5m"] = time_5m_marker,
	["1m"] = time_1m_marker,
	["30s"] = time_30s_marker,
	["15s"] = time_15s_marker,
	["5s"] = time_5s_marker,
	["1s"] = time_1s_marker
}

local time_marker = { 
	[300] = time_5m_marker,
	[60] = time_1m_marker,
	[30] = time_30s_marker,
	[15] = time_15s_marker,
	[5] = time_5s_marker,
	[1] = time_1s_marker
}


--Define local UI variables. 
--If something goes wrong with loading saved variables, these values wil be used.

--Logarithmic mode by default. If false, switches to linear.
local logarithmic = true

--Horizontal mode by default. If false, changes to vertical mode.
local horizontal = true

--Right-to-Left or Top-to-Bottom by default. If true, changes movement direction.
local movementFlipped = false

--Travel length is first width. Icon height is first height.
local travelLength = 400
local iconHeight = 20

--Position on the screen. 0, 0 is top left. 1, 1 is bottom right.
local xPos = 0.5
local yPos = 0.5

--Background Color. Red, Green, Blue from 0 to 1.
local bgRed = 0.5
local bgGreen = 0.3
local bgBlue = 0.3

--Max cooldown time
local maxCooldownTime = 600

--Store actual things on cooldown. Key = Ability ID. Value = Slider to show on bar.
local activeCooldowns = {}

--remember old sliders because we can't destroy them. All we have to do is change their texture.
local zombieSliders = {}

local function calcBarPoint(cooldownRemaining)
	if logarithmic then
		barPoint = (math.log(((cooldownRemaining / maxCooldownTime) * 1999) + 1) / math.log(2000))
	else
		barPoint = cooldownRemaining / maxCooldownTime
	end
	
	return barPoint
end
-- Overall UI Functions --

--Refreshes the UI in respect to the variables users can change.
--Runs when options are used to change the UI.
local function refreshUI()
	slideDownFrame:SetPoint("TOPCENTER", UIParent, xPos, yPos)
	
	actualHeight = 0
	actualLength = 0
	if horizontal then
		actualHeight = iconHeight
		actualLength = travelLength
		
		for timeAmount, marker in pairs(time_marker) do
			marker:SetPoint("CENTER", slideDownFrame, calcBarPoint(tonumber(timeAmount)), 0.5)
		end
	else
		actualHeight = travelLength
		actualLength = iconHeight
		
		for timeAmount, marker in pairs(time_marker) do
			marker:SetPoint("CENTER", slideDownFrame, 0.5, calcBarPoint(tonumber(timeAmount)))
		end
	end

	slideDownFrame:SetHeight(actualHeight)
	slideDownFrame:SetWidth(actualLength)
	
	for _, marker in pairs(time_marker) do
		marker:SetHeight(iconHeight)
		marker:SetWidth(iconHeight)
		marker:SetFontSize(iconHeight * 0.5)
	end
	
	slideDownFrame.bg:SetBackgroundColor(bgRed, bgGreen, bgBlue)
end

--Initial UI setup. Runs after saved variables are loaded.
local function setUpUI()
	--Set up the SlideDown main frame and background.
	slideDownFrame.bg = UI.CreateFrame("Frame", "slideDownFrame.bg", slideDownFrame)
	slideDownFrame.bg:SetPoint("TOPLEFT", slideDownFrame, "TOPLEFT")
	slideDownFrame.bg:SetPoint("BOTTOMRIGHT", slideDownFrame, "BOTTOMRIGHT")
	slideDownFrame.bg:SetLayer(-1)
	
	for label, marker in pairs(label_marker) do
		marker:SetText(label)
	end
	
	refreshUI()
	
	for _, marker in pairs(time_marker) do
		marker:SetVisible(true)
	end
	slideDownFrame.bg:SetVisible(true)
end



-- Saved Variables Handlers --

--Loads saved variables for different settings, if they exist.
--Then set up UI after loading.
local function loadVariables()
	if SlideDown_savedTravelLength then
		travelLength = SlideDown_savedTravelLength
	end
	if SlideDown_savedIconHeight then
		iconHeight = SlideDown_savedIconHeight
	end
	if SlideDown_savedMovementFlipped ~= nil then
		movementFlipped = SlideDown_savedMovementFlipped
	end
	if SlideDown_savedHorizontal ~= nil then
		horizontal = SlideDown_savedHorizontal
	end
	if SlideDown_savedBGRed then
		bgRed = SlideDown_savedBGRed
	end
	if SlideDown_savedBGGreen then
		bgGreen = SlideDown_savedBGGreen
	end
	if SlideDown_savedBGBlue then
		bgBlue = SlideDown_savedBGBlue
	end
	if SlideDown_savedXPos then
		xPos = SlideDown_savedXPos
	end
	if SlideDown_savedYPos then
		yPos = SlideDown_savedYPos
	end
	
	setUpUI()
end

--Saves variables for use again
local function saveVariables()
	
	print("saving")
	if travelLength then
		SlideDown_savedTravelLength = travelLength
	end
	if iconHeight then
		SlideDown_savedIconHeight = iconHeight
	end
	if movementFlipped ~= nil then
		SlideDown_savedMovementFlipped = movementFlipped
	end
	if horizontal ~= nil then
		SlideDown_savedHorizontal = horizontal
	end
	if bgRed then
		SlideDown_savedBGRed = bgRed
	end
	if bgGreen then
		SlideDown_savedBGGreen = bgGreen
	end
	if bgBlue then
		SlideDown_savedBGBlue = bgBlue
	end
	if xPos then
		SlideDown_savedXPos = xPos
	end
	if yPos then
		SlideDown_savedYPos = yPos
	end
end


-- Event Handlers --

--If the cooldown is real (not global cooldown), then make it a slider and store it in the active buff table.
--Runs for each cooldown in each Event.Ability.Cooldown.Begin.
local function cooldownBegin(abilityID, cooldownTime)
	detail = Inspect.Ability.Detail(abilityID)
	--Check to see if real cooldown or GCD
	if((detail.cooldown ~= nil) and 
		(detail.currentCooldown <= detail.cooldown + 0.0001) and
		(detail.currentCooldown >= detail.cooldown - 0.0001)) then
		--print("Cooldown Begin: " .. detail.name)
		
		
		--The icons that slide along the SlideDown bar are "sliders".
		--Currently, Sliders are squares as tall as the bar.
		
		--see if there is a zombie slider for us to use
		slider = table.remove(zombieSliders)
		if not slider then --make a new one if there wasn't a zombie available
			slider = UI.CreateFrame("Texture", "slider", slideDownFrame)
			slider:SetHeight(iconHeight)
			slider:SetWidth(iconHeight)
			--print("New Slider")
		else
			--We need to check to make sure our zombie slider is the correct size
			if slider:GetHeight() ~= iconHeight then
				slider:SetHeight(iconHeight)
				slider:SetWidth(iconHeight)
			end
		end
		slider:SetPoint("CENTER", slideDownFrame, "CENTERRIGHT")
		slider:SetVisible(false)
		slider:SetTexture("Rift", detail.icon)
		
		--put this slider into the active cooldown table
		activeCooldowns[abilityID] = slider
	end
end

--If the cooldown ending was active, remove it's slider and remove it from the active buffs table.
--Runs for each cooldown in each Event.Ability.Cooldown.End.
local function cooldownEnd(abilityID)
	--detail = Inspect.Ability.Detail(abilityID)
	--print("Cooldown End: " .. detail.name)
	
	--grab whatever is in the active cooldown table for this ID. 
	--It might be nil if this was triggered by the GCD.
	slider = activeCooldowns[abilityID]
	if slider then
		--hide it and make it a zombie.
		slider:SetVisible(false)
		table.insert(zombieSliders, slider)
		activeCooldowns[abilityID] = nil
	end
end


-- Refresh Functions --

--For each active cooldown, update it's slider.
--Runs each system update loop, for now.
local function slideDown()
	for abilityID, slider in pairs(activeCooldowns) do
		detail = Inspect.Ability.Detail(abilityID)
		if detail.currentCooldownRemaining then --check. Sometimes is null for last loop.
		
			cooldownRemaining = detail.currentCooldownRemaining
			if (cooldownRemaining > maxCooldownTime) then
				cooldownRemaining = maxCooldownTime
			end
			
			--print(detail.name)
			--barPoint defines the function which models the movement of the sliders.
			barPoint = calcBarPoint(cooldownRemaining)
			--print(barPoint)
			
			if movementFlipped then
				barPoint = 1.0 - barPoint
			end
			
			sliderXPos = 0
			sliderYPos = 0
			
			if horizontal then
				sliderXPos = barPoint
				sliderYPos = 0.5
			else
				sliderXPos = 0.5
				sliderYPos = barPoint
			end
			
			slider:SetPoint("CENTER", slideDownFrame, sliderXPos, sliderYPos)
			slider:SetVisible(true)
		end
	end
end



-- Slash Command Handlers --

--Distribute this request to it's proper function.
--Runs when a /slidedown command is entered
local function slashinput(input)
	--split everything past the first word
	splitIndex = string.find(input, ' ')
	if splitIndex then
		command = string.sub(input, 1, splitIndex - 1)
		args = string.sub(input, splitIndex + 1)
		--print("Command: " .. command)
		--print("Args: " .. args)
		if command == "pos" then
			split2 = string.find(args, ' ')
			if split2 then
				tempXPos = tonumber(string.sub(args, 1, split2 - 1))
				tempYPos = tonumber(string.sub(args, split2 + 1))
				if tempXPos and tempYPos then
					--print(xPos)
					--print(yPos)
					xPos = tempXPos
					yPos = tempYPos
					refreshUI()
				end
			end
		elseif command == "horiz" then
			if args == "false" then
				horizontal = false
			else
				horizontal = true
			end
			refreshUI()
		elseif command == "flipped" then
			if args == "false" then
				movementFlipped = false
			else
				movementFlipped = true
			end
		elseif command == "logarithmic" then
			if args == "false" then
				logarithmic = false
			else
				logarithmic = true
			end
		elseif command == "size" then
			split2 = string.find(args, ' ')
			if split2 then
				tempTrav = tonumber(string.sub(args, 1, split2 - 1))
				tempIcon = tonumber(string.sub(args, split2 + 1))
				if tempTrav and tempIcon then
					travelLength = tempTrav
					iconHeight = tempIcon
					refreshUI()
				end
			end
		elseif command == "color" then
			split2 = string.find(args, ' ')
			if split2 then
				red = tonumber(string.sub(args, 1, split2 - 1))
				tempGB = string.sub(args, split2 + 1)
				split3 = string.find(tempGB, ' ')
				if split3 then
					green = tonumber(string.sub(tempGB, 1, split3 - 1))
					blue = tonumber(string.sub(tempGB, split3 + 1))
					if red and green and blue then
						if (red <= 1.0 and red >= 0.0) and 
						(green <= 1.0 and green >= 0.0) and 
						(blue <= 1.0 and blue >= 0.0) then
							bgRed = red
							bgGreen = green
							bgBlue = blue
							refreshUI()
						end
					end
				end
			end
		end
	end
	
	if input == "reset" then
	logarithmic = true
	horizontal = true
	movementFlipped = false
	travelLength = 400
	iconHeight = 20
	xPos = 0.5
	yPos = 0.5
	bgRed = 0.5
	bgGreen = 0.3
	bgBlue = 0.3
	refreshUI()
	end
end

-- Event Hooks --

table.insert(Event.Addon.SavedVariables.Load.End, {loadVariables, "SlideDown", "Load saved variables"})
table.insert(Event.Addon.SavedVariables.Save.Begin, {saveVariables, "SlideDown", "Save saved variables"})

--Every frame, update the positions of active sliders.
table.insert(Event.System.Update.Begin, {slideDown, "SlideDown", "slideDown"})

--The cooldown begin event passes a table of all abilities that went on cooldown.
--key = ability ID, value = cooldown time remaining.
table.insert(Event.Ability.Cooldown.Begin,
	{
		function (params)
			for abilityID, cooldownTime in pairs(params) do
				cooldownBegin(abilityID, cooldownTime)
			end
		end, 
		"SlideDown", 
		"cooldownBegin"
	}
)

--The cooldown begin event passes a table of all abilities that came off cooldown.
--key = ability ID, value = cooldown time remaining (0, heh).
table.insert(Event.Ability.Cooldown.End, 
	{
		function (params)
			for abilityID, cooldownTime in pairs(params) do
				cooldownEnd(abilityID)
			end
		end, 
		"SlideDown", 
		"cooldownEnd"
	}
)

-- Slash Command Register --

table.insert(Command.Slash.Register("slidedown"), 
	{
		function (params)
			slashinput(params)
		end, 
		"SlideDown", 
		"Slash command providing access to SlideDown options."
	}
)